@@ -8,6 +8,7 @@ or invalidating entries easily.
 
 import json
 import collections
+import urllib
 
 from django.core.cache import cache as _cache
 
@@ -20,10 +21,7 @@ class CacheModel(object):
         pass
 
     def __init__(self, *args, **kwargs):
-        items = [(k, v) for (k, v)
-            in vars(type(self)).items()
-            if isinstance(v, Field)
-        ]
+        items = self._get_fields()
         known_fields = set()
         for name, field in items:
             if field.name is None:
@@ -38,6 +36,13 @@ class CacheModel(object):
             else:
                 raise AttributeError("Unknown field '{0}' given.".format(name))
 
+    @classmethod
+    def _get_fields(cls):
+        for k in dir(cls):
+            v = getattr(cls, k)
+            if isinstance(v, Field):
+                yield (k, v)
+
     def _all_keys(self):
         keys = self.keys()
         keys['$MODULE'] = type(self).__module__
@@ -51,8 +56,11 @@ class CacheModel(object):
             cache_key = getattr(type(self), k)
             return cache_key.order
         
+        items = [(k, getattr(type(self), k)) for k
+            in dir(type(self))
+        ]
         items = [(k, v) for (k, v)
-            in vars(type(self)).items()
+            in items
             if isinstance(v, Key)
         ]
 
@@ -60,6 +68,10 @@ class CacheModel(object):
             keys[k] = getattr(self, k)
         return keys
 
+    @classmethod
+    def _key_quote(cls, unquoted):
+        return unquoted.replace('_', '__').replace(' ', '_')
+
     @classmethod
     def _key(cls, keys):
         def sk(k, v):
@@ -68,7 +80,7 @@ class CacheModel(object):
             except AttributeError:
                 return v
             return '='.join((k, field.python_to_cache(v)))
-        return '/'.join(sk(k, v) for (k, v) in keys.items())
+        return cls._key_quote('/'.join(sk(k, v) for (k, v) in keys.items()))
 
     def key(self):
         return self._key(self._all_keys())
@@ -95,15 +107,27 @@ class CacheModel(object):
         _cache.set(key, s, expires)
 
     @classmethod
-    def get(cls, **kwargs):
+    def _get(cls, **kwargs):
         k = cls(**kwargs).key()
         data = _cache.get(k)
+        return data
+
+    @classmethod
+    def get(cls, **kwargs):
+        data = cls._get(**kwargs)
         if data is None:
             new = cls()
             new.from_miss(**kwargs)
             return new
         return cls.deserialize(data)
 
+    @classmethod
+    def get_or_create(cls, **kwargs):
+        data = cls._get(**kwargs)
+        if data is None:
+            return cls(**kwargs), True
+        return cls.deserialize(data), False
+
     def from_miss(self, **kwargs):
         """Called to initialize an instance when it is not found in the cache.
         
